﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace EduNetworkBuilder
{
    public class TraversalClass
    {
        protected struct TraversalRecord
        {
            public string host;
            public TraversalTechnology WhatUsed;

            public TraversalRecord(string Host, TraversalTechnology what)
            {
                host = Host;
                WhatUsed = what;
            }
        }

        List<TraversalRecord> PathTaken = new List<TraversalRecord>();
        public int Count { get { return PathTaken.Count; } }

        public TraversalClass(){ }

        public TraversalClass(TraversalClass CloneFrom)
        {
            Clone(CloneFrom, this);
        }

        public TraversalClass Clone()
        {
            TraversalClass newClass = new TraversalClass(this);
            return newClass;
        }

        public static void Clone(TraversalClass source, TraversalClass dest)
        {
            dest.PathTaken.AddRange(source.PathTaken);
        }

        public void AddPath(string Host, TraversalTechnology WithWhat)
        {
            PathTaken.Add(new TraversalRecord(Host, WithWhat));
        }
        public void AddPath(TraversalClass OtherTraversal)
        {
            PathTaken.AddRange(OtherTraversal.PathTaken);
        }
        /// <summary>
        /// This is just a debug function to print off the whole path as we find it.
        /// </summary>
        public void DumpPath()
        {
            foreach(TraversalRecord one in PathTaken)
            {
                Console.WriteLine(string.Format("{0:12} : {1:-12}",one.host, one.WhatUsed.ToString()));
            }
            Console.WriteLine(""); //blank line.
        }

        public string Destination()
        {
            //When we do a ping, only the return trip is registered.  source/dest are reversed
            if (PathTaken.Count >= 1) return PathTaken[0].host;
            return "";
        }

        public string Source()
        {
            //When we do a ping, only the return trip is registered.  source/dest are reversed
            if (PathTaken.Count >= 1) return PathTaken[PathTaken.Count -1].host;
            return "";
        }

        public string HostnameFromTechnology(TraversalTechnology What)
        {             
            //make a randomized copy of the list, and return the first item that has the technology
            List<TraversalRecord> tPathTaken= NB.Randomize<TraversalRecord>(PathTaken);
            foreach(TraversalRecord one in tPathTaken)
            {
                if (one.WhatUsed == What) return one.host;
            }
            return "";
        }
    }
}
